@@ -15,7 +15,16 @@
 """
 import unittest
 
+from zope.interface.tests.test_adapter import CustomTypesBaseAdapterRegistryTests
 
+def skipIfNoPersistent(testfunc):
+    try:
+        import persistent
+    except ImportError: # pragma: no cover
+        return unittest.skip("persistent not installed")(testfunc)
+    return testfunc
+
+@skipIfNoPersistent
 class PersistentAdapterRegistryTests(unittest.TestCase):
 
     def _getTargetClass(self):
@@ -32,13 +41,13 @@ class PersistentAdapterRegistryTests(unittest.TestCase):
             def __init__(self, jar):
                 self._jar = jar
                 self._mru = []
-            def mru(self, oid):
-                self._mru.append(oid)
+                # mru(oid) is only called in pure-Python runs
+                self.mru = self._mru.append
             def new_ghost(self, oid, obj):
                 obj._p_jar = self._jar
                 obj._p_oid = oid
             def update_object_size_estimation(self, oid, size):
-                return
+                "This is only called in pure-Python runs"
 
         return _Cache(jar)
 
@@ -93,12 +102,13 @@ class PersistentAdapterRegistryTests(unittest.TestCase):
         bases = (globalSiteManager.adapters, globalSiteManager.utilities)
         registry, jar, OID = self._makeOneWithJar(bases=bases)
         state = registry.__getstate__()
-        self.assertEqual(state['__bases__'], bases)
-        self.assertEqual(state['_generation'], 1)
-        self.assertEqual(state['_provided'], {})
-        self.assertEqual(state['_adapters'], [])
-        self.assertEqual(state['_subscribers'], [])
-        self.assertFalse('ro' in state)
+        self.assertEqual(state.pop('__bases__'), bases)
+        self.assertEqual(state.pop('_generation'), 1)
+        self.assertEqual(state.pop('_provided'), {})
+        self.assertEqual(state.pop('_adapters'), [])
+        self.assertEqual(state.pop('_subscribers'), [])
+        self.assertNotIn('ro', state)
+        self.assertEqual(state, {})
 
     def test___getstate___skips_delegated_names(self):
         registry, jar, OID = self._makeOneWithJar()
@@ -123,7 +133,15 @@ class PersistentAdapterRegistryTests(unittest.TestCase):
         self.assertEqual(registry.__bases__, bases)
         self.assertEqual(registry.ro, [registry] + list(bases))
 
+    def test__addValueToLeaf_existing_is_tuple_converts(self):
+        from persistent.list import PersistentList
+        registry = self._makeOne()
+        result = registry._addValueToLeaf(('a',), 'b')
+        self.assertIsInstance(result, PersistentList)
+        self.assertEqual(result, ['a', 'b'])
 
+
+@skipIfNoPersistent
 class PersistentComponentsTests(unittest.TestCase):
 
     def _getTargetClass(self):
@@ -152,13 +170,26 @@ class PersistentComponentsTests(unittest.TestCase):
                                    PersistentList))
 
 def _makeOctets(s):
-    import sys
-    if sys.version_info < (3,):
-        return bytes(s)
-    return bytes(s, 'ascii') #pragma NO COVERAGE
+    return bytes(s) if bytes is str else bytes(s, 'ascii')
 
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(PersistentAdapterRegistryTests),
-        unittest.makeSuite(PersistentComponentsTests),
-    ))
+
+@skipIfNoPersistent
+class PersistentAdapterRegistryCustomTypesTest(CustomTypesBaseAdapterRegistryTests):
+
+    def _getMappingType(self):
+        from persistent.mapping import PersistentMapping
+        return PersistentMapping
+
+    def _getProvidedType(self):
+        return self._getMappingType()
+
+    def _getMutableListType(self):
+        from persistent.list import PersistentList
+        return PersistentList
+
+    def _getLeafSequenceType(self):
+        return self._getMutableListType()
+
+    def _getBaseAdapterRegistry(self):
+        from zope.component.persistentregistry import PersistentAdapterRegistry
+        return PersistentAdapterRegistry
